38#include <hip/hip_runtime.h>
50 void *
dx,
void *
dy,
void *
dz,
62 void *
dx,
void *
dy,
void *
dz,
66 void *
w3,
int *nel,
int *lx) {
78#define CASE_1D(LX, C) \
79 hipLaunchKernelGGL( HIP_KERNEL_NAME( \
80 opgrad_kernel_1d<real, LX, NEKO_CHUNKS(LX, C)> ), \
81 nblcks, NEKO_CHUNKS_NTHRDS(LX, C), 0, \
82 (hipStream_t) glb_cmd_queue, \
83 (real *) ux, (real *) uy, (real *) uz, (real *) u, \
84 (real *) dx, (real *) dy, (real *) dz, \
85 (real *) drdx, (real *) dsdx, (real *) dtdx, \
86 (real *) drdy, (real *) dsdy, (real *) dtdy, \
87 (real *) drdz, (real *) dsdz, (real *) dtdz, \
89 HIP_CHECK(hipGetLastError());
92#define CASE_1D_SEL(LX, SEL) \
94 case 0: CASE_1D(LX, 0); break; \
95 case 1: CASE_1D(LX, 1); break; \
96 case 2: CASE_1D(LX, 2); break; \
97 default: CASE_1D(LX, 3); break; \
101#define CASE_KSTEP(LX, C) \
102 hipLaunchKernelGGL( HIP_KERNEL_NAME( \
103 opgrad_kernel_kstep<real, LX, NEKO_EB(LX, C)> ), \
104 NEKO_EB_NBLCKS(*nel, LX, C), NEKO_EB_NTHRDS(LX, C), 0, \
105 (hipStream_t) glb_cmd_queue, \
106 (real *) ux, (real *) uy, (real *) uz, (real *) u, \
107 (real *) dx, (real *) dy, (real *) dz, \
108 (real *) drdx, (real *) dsdx, (real *) dtdx, \
109 (real *) drdy, (real *) dsdy, (real *) dtdy, \
110 (real *) drdz, (real *) dsdz, (real *) dtdz, \
111 (real *) w3, *nel); \
112 HIP_CHECK(hipGetLastError());
115#define CASE_KSTEP_SEL(LX, SEL) \
117 case 0: CASE_KSTEP(LX, 0); break; \
118 case 1: CASE_KSTEP(LX, 1); break; \
119 default: CASE_KSTEP(LX, 2); break; \
124 if(autotune[LX] == 0 ) { \
125 autotune[LX]=tune_opgrad<LX>(ux, uy, uz, u, \
130 w3, nel, lx, &autotune_eb[LX], \
132 } else if (autotune[LX] == 1 ) { \
133 CASE_1D_SEL(LX, autotune_ch[LX]); \
134 } else if (autotune[LX] == 2 ) { \
135 CASE_KSTEP_SEL(LX, autotune_eb[LX]); \
166template < const
int LX >
168 void *
dx,
void *
dy,
void *
dz,
251 for (
int r = 0; r <
rounds; r++) {
__global__ void ale_add_kinematics_kernel(const int n, T *__restrict__ wx, T *__restrict__ wy, T *__restrict__ wz, const T *__restrict__ x_ref, const T *__restrict__ y_ref, const T *__restrict__ z_ref, const T *__restrict__ phi, const T *__restrict__ x, const T *__restrict__ y, const T *__restrict__ z, const kinematics_params_t kin_params)
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ drdy
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ drdz
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dsdz
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dsdy
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dtdy
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ u
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dx
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ drdx
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dtdz
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dsdx
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dtdx
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dz
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ dy
__global__ void const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ w3
#define NEKO_CHUNKS_CANDIDATES
#define NEKO_EB_CANDIDATES
#define NEKO_EB_SEL(LX, SEL)
#define NEKO_CHUNKS_SEL(LX, SEL)
#define NEKO_TUNE_TIME(T, LAUNCH, LX, C, ITERS)
static int neko_tune_rounds()
#define NEKO_TUNE_LOG(LX, T1, T2)
#define NEKO_TUNE_BEST(T, BEST, N)
static int neko_tune_iters()
static int neko_chunks_env()
static int neko_eb_sweep()
__global__ void T *__restrict__ uy
__global__ void T *__restrict__ T *__restrict__ uz
void log_error(char *msg)
void log_message(char *msg)
void log_section(char *msg)
#define CASE_KSTEP_SEL(LX, SEL)
#define CASE_1D_SEL(LX, SEL)
#define CASE_KSTEP(LX, C)
int tune_opgrad(void *ux, void *uy, void *uz, void *u, void *dx, void *dy, void *dz, void *drdx, void *dsdx, void *dtdx, void *drdy, void *dsdy, void *dtdy, void *drdz, void *dsdz, void *dtdz, void *w3, int *nel, int *lx, int *eb_sel, int *ch_sel)
void hip_opgrad(void *ux, void *uy, void *uz, void *u, void *dx, void *dy, void *dz, void *drdx, void *dsdx, void *dtdx, void *drdy, void *dsdy, void *dtdy, void *drdz, void *dsdz, void *dtdz, void *w3, int *nel, int *lx)